@@ -16,30 +16,40 @@ import pickle
 import sys
 import time
 
+try:
+    import deepdish
+except ModuleNotFoundError:
+    print("\033[91m")
+    print("ModuleNotFoundError", end="")
+    print("\033[0m", end="")
+    print(": No module named 'deepdish'")
+    print()
+    print("Skipping this import and continuing.")
+
 import nds2
 import numpy as np
 import scipy.constants as scc
 import scipy.signal as sig
 from scipy.optimize import fsolve
 
-"""Functions"""
+
 # dB conversions
-def dB2Mag(dBnum):
+def db_to_mag(db):
     """Takes in number in dB, returns magnitude = 10^(dB/20)"""
-    return 10 ** (dBnum / 20.0)
+    return 10 ** (db / 20.0)
 
 
-def mag2dB(magNum):
+def mag_to_db(mag):
     """Takes in number in magnitude, returns in dB = 20*log10(mag)"""
-    return 20.0 * np.log10(magNum)
+    return 20.0 * np.log10(mag)
 
 
 # Complex Algebra
-def QMag2ComplexPoles(Q, f0):
+def complex_pole_pair(qq, f0):
     """Takes in Q and f0 (center frequency in Hz) and returns a complex pole pair"""
-    f0_2 = f0 ** 2
-    real = 0.5 / Q
-    imag = np.sqrt(f0_2 - real ** 2)
+    f0_2 = f0**2
+    real = 0.5 / qq
+    imag = np.sqrt(f0_2 - real**2)
     poles = np.array([real + 1j * imag, real - 1j * imag])
     return poles
 
@@ -49,7 +59,7 @@ def good_ticks(axis):
     """Finds the plot range, and sets the y ticks to always by factors of 10 on the y scale"""
     ymin, ymax = axis.get_ylim()
     yTicks = np.array(
-        [10 ** x for x in np.arange(np.ceil(np.log10(ymin)), np.ceil(np.log10(ymax)))]
+        [10**x for x in np.arange(np.ceil(np.log10(ymin)), np.ceil(np.log10(ymax)))]
     )
     return yTicks
 
@@ -69,6 +79,24 @@ def load_pickle(savefile):
     return dataDict
 
 
+### .hdf5 file save and load using deepdish
+def save_hdf5(dataDict, savefile):
+    """Saves a .hdf5 file using deepdish,
+    use blosc compression to greatly enhance speed"""
+    deepdish.io.save(savefile, dataDict, compression="blosc")
+    return
+
+
+def load_hdf5(loadfile, group=None):
+    """Loads an hdf5 file using deepdish"""
+    if not group:
+        data_dict = deepdish.io.load(loadfile)
+    else:
+        data_dict = deepdish.io.load(loadfile, group=group)
+
+    return data_dict
+
+
 # FFT parameters
 def dtt_time(averages, bandwidth, overlap, verbose=False):
     """
@@ -111,6 +139,28 @@ def dtt_averages(time, bandwidth, overlap, verbose=False):
     return averages
 
 
+def minimum_coherent_power(averages, overlap):
+    """Given the number of averages taken and the overlap between FFTs,
+    returns the expected value of the minimum measureable coherent power between two signals.
+
+    In other words, if you compute the coherence between two completely uncorrelated signals a and b,
+    then the coherence function <a,b><b,a>/<a,a><b,b> will return values around this number.
+    A measured coherence considerably above this number can be considered resolved.
+
+    This is equal to 1/(averages * (1 - overlap))
+    In reality, we will see an extremely noisy curve
+    """
+    if overlap > 1.0:  # overlap is in percent
+        overlap = overlap / 100.0
+    if overlap < 0.0 or overlap > 1.0:  # if overlap is still not in range
+        print("overlap is not valid, must be between 0 and 100 percent, or 0 and 1.")
+        print("overlap = {} %".format(overlap * 100.0))
+        return
+
+    min_expected_coherent_power = 1.0 / (averages * (1 - overlap))
+    return min_expected_coherent_power
+
+
 # Data acquistion/storage/manipulation
 def find_channels(
     channel_glob="*",
@@ -667,13 +717,14 @@ def stitch_real_time_data(
 
     for chan in dataDict.keys():
         data = dataDict[chan]["data"]
-        times = np.arange(len(dataDict[chan]["data"])) / float(dataDict[chan]["fs"])
+        fs = dataDict[chan]["fs"]
 
-        goodIndices = np.argwhere(times < duration)[:, 0]
-        # goodTimes = times[goodIndices]
-        goodData = data[goodIndices]
+        nn = len(data)
+        total_time = nn / fs
+        fraction_time = duration / total_time
+        cutoff_index = int(fraction_time * nn)
 
-        # dataDict[chan]['times'] = goodTimes
+        goodData = data[:cutoff_index]
         dataDict[chan]["data"] = goodData
 
     return dataDict
@@ -695,13 +746,14 @@ def extract_dict(chanData, duration):
 
     for chan in dataDict.keys():
         data = dataDict[chan]["data"]
-        times = np.arange(len(dataDict[chan]["data"])) / float(dataDict[chan]["fs"])
+        fs = dataDict[chan]["fs"]
 
-        goodIndices = np.argwhere(times < duration)[:, 0]
-        goodTimes = times[goodIndices]
-        goodData = data[goodIndices]
+        nn = len(data)
+        total_time = nn / fs
+        fraction_time = duration / total_time
+        cutoff_index = int(fraction_time * nn)
 
-        # dataDict[chan]['times'] = goodTimes # half the cost of data storage
+        goodData = data[:cutoff_index]
         dataDict[chan]["data"] = goodData
 
     return dataDict
@@ -713,13 +765,15 @@ def trim_dict(dataDict, duration):
     """
     for chan in dataDict.keys():
         data = dataDict[chan]["data"]
-        times = dataDict[chan]["times"]
+        fs = dataDict[chan]["fs"]
 
-        goodIndices = np.argwhere(times < duration)[:, 0]
-        goodTimes = times[goodIndices]
-        goodData = data[goodIndices]
+        nn = len(data)
+        total_time = nn / fs
+        fraction_time = duration / total_time
+        cutoff_index = int(fraction_time * nn)
+
+        goodData = data[:cutoff_index]
 
-        dataDict[chan]["times"] = goodTimes
         dataDict[chan]["data"] = goodData
 
     return dataDict
@@ -1511,7 +1565,7 @@ def median_coherence_from_power_ratio(power_ratio):
     power_ratio = sigma_uncorr^2 / sigma_corr^2,
     returns the power coherence gamma^2 that would result from median-averaged PSDs and CSDs.
     """
-    med_coh = (power_ratio ** 2 * np.log(1 + 1 / np.sqrt(1 + power_ratio)) ** 2) / (
+    med_coh = (power_ratio**2 * np.log(1 + 1 / np.sqrt(1 + power_ratio)) ** 2) / (
         4
         * np.log(2) ** 2
         * (1 + power_ratio)
@@ -1697,7 +1751,7 @@ def RMS(freq, spec):
     """Takes in frequency vector and ASD, returns the RMS from high to low freq"""
     df = np.ediff1d(freq)
     df = np.append(df, df[-1])
-    return np.flipud(np.sqrt(np.cumsum(np.flipud(df * spec ** 2))))
+    return np.flipud(np.sqrt(np.cumsum(np.flipud(df * spec**2))))
 
 
 def simple_OLG(fff, UGF):
